
#include <stdafx.h>
#include "windows.h"
#include "shellapi.h"



#include <stdio.h>
#include <stdarg.h>


#include "os.h"

#include "deb.h"

#include "err.h"
#include "oserr.h"


static os_error errblock;



os_error * XGetLastError2(char * name,char * format,...)
{
 va_list   args;
 char      v[512];
 int       code;

 code=GetLastError();
 if(code==ERROR_SUCCESS)
 {
  return(NULL);
 }
 else
 {
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
    NULL,code,0,errblock.errmess,sizeof(errblock.errmess),NULL);

  errblock.errnum=code;

  if(format)
  {
   va_start(args, format);
   vsprintf(v, format, args);

   sprintf(errblock.errmess+strlen(errblock.errmess),"(%s) (%d) (%s)",name,code,v);

   va_end(args);
  }
  else
  {
   sprintf(errblock.errmess+strlen(errblock.errmess),"(%s) (%d)",name,code);
  }

  return(&errblock);
 }
}



// have another version that does not look for error success
// this is because on '98 at start up FindResource returns a null
// handle if a resource is not present, but getlasterror seems to
// give a success code


os_error * XGetLastError2N(char * name,char * format,...)
{
 va_list   args;
 char      v[512];
 int       code;

 code=GetLastError();

 {
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
    NULL,code,0,errblock.errmess,sizeof(errblock.errmess),NULL);

  errblock.errnum=code;

  if(format)
  {
   va_start(args, format);
   vsprintf(v, format, args);

   sprintf(errblock.errmess+strlen(errblock.errmess),"(%s) (%d) (%s)",name,code,v);

   va_end(args);
  }
  else
  {
   sprintf(errblock.errmess+strlen(errblock.errmess),"(%s) (%d)",name,code);
  }

  return(&errblock);
 }
}









os_error * XCloseHandle(
  HANDLE hObject   // handle to object to close
)
{
 int code;

 code=CloseHandle(hObject);

 if(code==0) return(XGetLastError2("CloseHandle",NULL));
 return(NULL);
}




os_error * XSetFilePointer(
  HANDLE hFile,          // handle of file
  LONG lDistanceToMove,  // number of bytes to move file pointer
  PLONG lpDistanceToMoveHigh,
                         // pointer to high-order DWORD of 
                         // distance to move
  DWORD dwMoveMethod,    // how to move
  DWORD * result
)

{
 os_error * err;

 *result=SetFilePointer(hFile,lDistanceToMove,lpDistanceToMoveHigh,dwMoveMethod);
 if(*result==0xFFFFFFFF)
 {
  err=XGetLastError2("SetFilePointer","");
  if(lpDistanceToMoveHigh)
  {
   if(err->errnum==NO_ERROR) return(NULL);
  }
  return(err);
 }
 return(NULL);
}



os_error * XReadFile(
  HANDLE hFile,                // handle of file to read
  LPVOID lpBuffer,             // pointer to buffer that receives data
  DWORD nNumberOfBytesToRead,  // number of bytes to read
  LPDWORD lpNumberOfBytesRead, // pointer to number of bytes read
  LPOVERLAPPED lpOverlapped    // pointer to structure for data
)
{
 int code;

 code=ReadFile(hFile,lpBuffer,nNumberOfBytesToRead,lpNumberOfBytesRead,lpOverlapped);

 if(code==0) return(XGetLastError2("ReadFile",NULL));
 return(NULL);
}



os_error * XWriteFile(
  HANDLE hFile,                    // handle to file to write to
  LPCVOID lpBuffer,                // pointer to data to write to file
  DWORD nNumberOfBytesToWrite,     // number of bytes to write
  LPDWORD lpNumberOfBytesWritten,  // pointer to number of bytes written
  LPOVERLAPPED lpOverlapped        // pointer to structure for overlapped I/O
)
{
 int code;

 code=WriteFile(hFile,lpBuffer,nNumberOfBytesToWrite,lpNumberOfBytesWritten,lpOverlapped);
 if(code==0) return(XGetLastError2("WriteFile",NULL));
 return(NULL);
}


os_error * XGetFileAttributes(
  LPCTSTR lpFileName,   // pointer to the name of a file or directory
  DWORD * result
)
{
 DWORD code;

 code=GetFileAttributes(lpFileName);
 if(code==0xFFFFFFFF) return(XGetLastError2("GetFileAttributes","%s",lpFileName));
 *result=code;
 return(NULL);
}



os_error * XCreateFile(
  LPCTSTR lpFileName,          // pointer to name of the file
  DWORD dwDesiredAccess,       // access (read-write) mode
  DWORD dwShareMode,           // share mode
  LPSECURITY_ATTRIBUTES lpSecurityAttributes,
                               // pointer to security attributes
  DWORD dwCreationDisposition,  // how to create
  DWORD dwFlagsAndAttributes,  // file attributes
  HANDLE hTemplateFile,         // handle to file with attributes to 
                               // copy
  HANDLE * filehandle
)
 
{

 *filehandle=CreateFile(lpFileName,dwDesiredAccess,dwShareMode,
                        lpSecurityAttributes,dwCreationDisposition,
                        dwFlagsAndAttributes,hTemplateFile);
 
 if(*filehandle==INVALID_HANDLE_VALUE) return(XGetLastError2("CreateFile","%s",lpFileName));
 return(NULL);
}







os_error *  XFindResource(
  HMODULE hModule, // module handle
  LPCTSTR lpName,  // pointer to resource name
  LPCTSTR lpType,   // pointer to resource type
  HRSRC * handle
)
{
 
 *handle=FindResource(hModule,lpName,lpType);

 if(*handle==NULL) return(XGetLastError2N("FindResource","(%s)",lpName));
 return(NULL);
}


os_error * XLoadResource(
  HMODULE hModule, // resource-module handle
  HRSRC hResInfo,   // resource handle
  HGLOBAL * handle
)
{
 
 *handle=LoadResource(hModule,hResInfo);

 if(*handle==NULL) return(XGetLastError2("LoadResource",NULL));
 return(NULL);
}



os_error * XLockResource(
  HGLOBAL hResData,
  LPVOID * handle
)
{

 *handle=LockResource(hResData );
 return(NULL);
}


os_error * XSizeofResource(
  HMODULE hModule, // resource-module handle
  HRSRC hResInfo,   // resource handle
  unsigned int * length
)
{
 *length=SizeofResource(hModule,hResInfo);
 if(*length==NULL) return(XGetLastError2("SizeofResource",NULL));
 return(NULL);
} 
